;;; sporshe.lisp --- The Porshe class with stateful visits ala C++.

;; Author:        Didier Verna <didier@lrde.epita.fr>
;; Maintainer:    Didier Verna <didier@lrde.epita.fr>
;; Created:       Tue Apr  7 16:54:18 2009
;; Last Revision: Tue Apr  7 17:29:03 2009


;;; Commentary:

;; Contents management by FCM version 0.1.


;;; Code:

;; If we want to visit with state, a better way to do it is to get back to
;; visitor OBJECTS, so let's get back to that first. Note that I have removed
;; the VISITABLE-CAR-COMPONENT class.

(defclass porshe-visitor ()
  ())

(defclass wheel ()
  ())

(defclass body ()
  ((wheels :initform (loop :for i :from 1 :upto 4
			   :for wheel = (make-instance 'wheel)
			   :collect wheel)
	   :accessor wheels)))

(defclass engine ()
  ())

(defclass porshe ()
  ((engine :initform (make-instance 'engine)
	   :accessor engine)
   (body :initform (make-instance 'body)
	 :accessor body)))


(defgeneric visit (object visitor))

(defgeneric accept (object visitor)
  (:method ((wheel wheel) (visitor porshe-visitor))
    (visit wheel visitor))
  (:method ((body body) (visitor porshe-visitor))
    (visit body visitor)
    (dolist (wheel (wheels body))
      (accept wheel visitor)))
  (:method ((engine engine) (visitor porshe-visitor))
    (visit engine visitor))
  (:method ((porshe porshe) (visitor porshe-visitor))
    (visit porshe visitor)
    (accept (engine porshe) visitor)
    (accept (body porshe) visitor)))


;; A counter visitor.
(defclass counter-porshe-visitor (porshe-visitor)
  ((counter :initform 0 :accessor counter)))

;; And the method for all objects:
(defmethod visit (object (visitor counter-porshe-visitor))
  (incf (counter visitor)))


(defun main ()
  (let ((porshe (make-instance 'porshe))
	(visitor (make-instance 'counter-porshe-visitor)))
    (accept porshe visitor)
    (format t "The Porshe has ~S components.~%" (counter visitor)))
  (quit))

;; OK, so this works, but CRAP. Having to go back to visitor objects
;; machinery, just to perform a stupid counter increment ?! There has to be a
;; better way.


;;; sporshe.lisp ends here
