;;; sporshelmbcls.lisp --- The Porshe class with stateful lambda visits.

;; Author:        Didier Verna <didier@lrde.epita.fr>
;; Maintainer:    Didier Verna <didier@lrde.epita.fr>
;; Created:       Tue Apr  7 11:23:25 2009
;; Last Revision: Tue Apr  7 17:44:37 2009


;;; Commentary:

;; Contents management by FCM version 0.1.


;;; Code:


(defgeneric mapobject (func object)
  (:method (func (object standard-object))
    (funcall func  object)
    (mapc
     (lambda (slot)
       (funcall #'mapobject func
		(slot-value object (sb-mop:slot-definition-name slot))))
     (sb-mop:class-direct-slots (class-of object))))
  (:method (func (object list))
    (mapc func object)))


(defclass wheel ()
  ())

(defclass body ()
  ((wheels :initform (loop :for i :from 1 :upto 4
			   :for wheel = (make-instance 'wheel)
			   :collect wheel)
	   :accessor wheels)))

(defclass engine ()
  ())

(defclass porshe ()
  ((engine :initform (make-instance 'engine)
	   :accessor engine)
   (body :initform (make-instance 'body)
	 :accessor body)))

;; This time, we're going to use a lexical closure on a counter variable, and
;; a lambda (anonymous) function to perform the increment.
(defun main ()
  (let ((porshe (make-instance 'porshe))
	(counter 0))
    (mapobject (lambda (obj)
		 (declare (ignore obj))
		 (incf counter))
	       porshe)
    (format t "The Porshe has ~S components.~%" counter))
  (quit))


;;; sporshelmbcls.lisp ends here
