;;; vporshemap.lisp --- Making the idea of "mapping" explicit.

;; Author:        Didier Verna <didier@lrde.epita.fr>
;; Maintainer:    Didier Verna <didier@lrde.epita.fr>
;; Created:       Tue Apr  7 11:23:25 2009
;; Last Revision: Tue Apr  7 11:23:25 2009


;;; Commentary:

;; Contents management by FCM version 0.1.


;;; Code:


;; Visiting is really some kind of a structural mapping:
(defgeneric mapobject (func object)
  (:method (func object)
    (funcall func  object)))


(defclass wheel ()
  ())

(defclass body ()
  ((wheels :initform (loop :for i :from 1 :upto 4
			   :for wheel = (make-instance 'wheel)
			   :collect wheel)
	   :accessor wheels)))

(defmethod mapobject (func (body body))
  (funcall func body)
  (dolist (wheel (wheels body))
    (mapobject func wheel)))

(defclass engine ()
  ())

(defclass porshe ()
  ((engine :initform (make-instance 'engine)
	   :accessor engine)
   (body :initform (make-instance 'body)
	 :accessor body)))

(defmethod mapobject (func (porshe porshe))
  (funcall func porshe)
  (mapobject func (engine porshe))
  (mapobject func (body porshe)))

(defgeneric paint (object)
  (:method ((wheel wheel))
    (format t "Painting a wheel.~%"))
  (:method ((body body))
    (format t "Painting the body.~%"))
  (:method ((engine engine))
    (format t "Painting the engine.~%"))
  (:method ((porshe porshe))
    (format t "Painting the Porshe.~%")))

(defgeneric recycle (object)
  (:method ((wheel wheel))
    (format t "Recycling a wheel.~%"))
  (:method ((body body))
    (format t "Recycling the body.~%"))
  (:method ((engine engine))
    (format t "Recycling the engine.~%"))
  (:method ((porshe porshe))
    (format t "Recycling the Porshe.~%")))


(defun main ()
  (let ((porshe (make-instance 'porshe)))
    (mapobject #'paint porshe)
    (mapobject #'recycle porshe))
  (quit))


;;; vporshemap.lisp ends here
